/******************************************************************************/
/*                                                                            */
/*    	     FICHIER D'EXTRACTION DE LAYOUT                                   */
/*                                                                            */
/******************************************************************************/

drcExtractRules(

; Define all the layout and a variable to represent it

	L_NWELL = geomOr( "L_NWELL" )
  	L_PWELL = geomOr( "L_PWELL" ) 
	L_NDIF  = geomOr( "L_NDIF" )
	L_PDIF  = geomOr( "L_PDIF" )
  	L_NTIE  = geomOr( "L_NTIE" )
  	L_PTIE  = geomOr( "L_PTIE" )
  	L_POLY  = geomOr( "L_POLY" )
  	L_GATE  = geomOr( "L_GATE" )
	L_TPOLY = geomOr( "L_TPOLY" )
  	L_ALU1  = geomOr( "L_ALU1" )
  	L_TALU1 = geomOr( "L_TALU1" )
  	L_VIA1  = geomOr( "L_VIA1" )
  	L_ALU2  = geomOr( "L_ALU2" )
  	L_TALU2 = geomOr( "L_TALU2" )
  	L_VIA2  = geomOr( "L_VIA2" )
  	L_ALU3  = geomOr( "L_ALU3" )
  	L_TALU3 = geomOr( "L_TALU3" )
	L_CONT  = geomOr( "L_CONT" )
	L_REF_C  = geomOr( "L_REF_C" )

/********************************************************************/
/*                                                                  */
/*        TRANSISTOR EXTRAXT (LPE) FOR A SPICE NETLIST 		    */
/*                                                                  */
/********************************************************************/
ivIf( switch("extract?") then
	ivIf( switch("LPE_CADAL") || switch("LPE_LYNX") then
	
; Define a bulk for pwell it is only this layout doesn't really exist
; But it's usefull to extract the transistors

	pwell = geomNot( L_NWELL )

; These four layout above are for the extractor to recognise a transistor instance
	
	pgate = geomAnd( L_GATE L_PDIF )
	ngate = geomAnd( L_GATE L_NDIF )

	psd = geomAndNot( L_PDIF L_GATE )
	nsd = geomAndNot( L_NDIF L_GATE )

; These layout are the connection between layout 

	cont_gate = geomAnd( L_GATE L_POLY )
	cont_nwell = geomAnd( L_NTIE L_NWELL)
	cont_pwell = geomAnd( L_PTIE pwell)
        
	ncont = geomAnd( L_NDIF L_NTIE )
	pcont = geomAnd( L_PDIF L_PTIE )

; This function define the connection.
; The first layer is the contact
; the two or more other define which layers are connected

	geomConnect(
	via( cont_gate L_GATE L_POLY )

	via( cont_nwell L_NTIE L_NWELL )
	via( cont_pwell L_PTIE pwell )

	via( L_CONT psd  L_ALU1 )
	via( L_CONT nsd  L_ALU1 )
	via( L_CONT L_NTIE  L_ALU1 L_PTIE L_POLY )

	via( L_VIA1 L_ALU1 L_ALU2 )

	via( ncont L_NDIF L_NTIE )
	via( pcont L_PDIF L_PTIE )
	)

; Extract the transistor "NTRANS symbol" call for the cell vue in pcell

	extractMOS( ngate (L_GATE "G") (nsd "D" "S") ( pwell   "B") "NTRANS symbol" )
	extractMOS( pgate (L_GATE "G") (psd "D" "S") ( L_NWELL "B") "PTRANS symbol" )

; Save the recognition of the transistor

	saveRecognition( ngate "device")
	saveRecognition( pgate "device")
; If LPE_CADAL switch have been chosen
	ivIf( switch("LPE_CADAL") then

	;       NMOS capa drain-source area between two nmos is divided by 2
	; 	Or kept for a single PMOS
	  
	  a = measureParasitic( area ( nsd ) figure )
	  attachParasitic( a ("AS" "S") ("AD" "D") ngate shared )

	  b = measureParasitic( length ( nsd outside L_POLY ) figure )
	  attachParasitic( b ("PS" "S") ("PD" "D") ngate shared )
	  
	;       PMOS capa drain-source area between two pmos is divided by 2
	; 	Or kept for a single PMOS
	  
	  c = measureParasitic( area ( psd ) figure )
	  attachParasitic( c ("AS" "S") ("AD" "D") pgate shared )

	  d = measureParasitic( length ( psd outside L_POLY ) figure )
	  attachParasitic( d ("PS" "S") ("PD" "D") pgate shared )

	;	Capacitance calculation

	;	the value at the end is the surface capacitance
	;	or the perimetric capacitance

	cap1a  = measureParasitic(area ( L_POLY ) two_net 1.000000e-4)
	cap1p  = measureParasitic(perimeter ( L_POLY ) two_net 1.000000e-4)
	cap2a  = measureParasitic(area ( L_ALU1 ) two_net 0.500000e-4)
	cap2p  = measureParasitic(perimeter ( L_ALU1 ) two_net 0.900000e-4)
	cap3a  = measureParasitic(area ( L_ALU2 ) two_net 0.250000e-4)
	cap3p  = measureParasitic(perimeter ( L_ALU2 ) two_net 0.950000e-4)

	; All the capacitance are added and save as a symbol in the extract

	cap  = calculateParasitic( cap1a + cap1p + cap2a + cap2p + cap3a + cap3p )

	saveParasitic(cap   "PLUS" "MINUS" "c" "PCAPA symbol")
  	)


	ivIf( switch("LPE_LYNX") then

	;       NMOS capa drain-source each area is calculate from a nmos device


	; to define the size, we use the fact that a transistor is always 2' wide
	; This is why we multiply by 2 the length for the area
	; and we multiply by 2 and add 4 for the perimeter

	distN = measureParameter( length ( ngate butting nsd ) )
	saveParameter( distN "AS" )
	saveParameter( distN "AD" )

	perimN = calculateParameter( distN+4 )
	saveParameter( perimN "PS" )
	saveParameter( perimN "PD" )

	;       PMOS capa drain-source each area is calculate from a pmos device

	distP = measureParameter( length ( pgate butting psd ) )
	saveParameter( distP "AS" )
	saveParameter( distP "AD" )

	perimP = calculateParameter( distP+4 )
	saveParameter( perimP "PS" )
	saveParameter( perimP "PD" )

	;	Capacitance calculation
	;	The calculation is the same as for LPE_CADAL

	cap1a  = measureParasitic(area ( L_POLY ) two_net 1.000000e-4)
	cap1p  = measureParasitic(perimeter ( L_POLY ) two_net 1.000000e-4)
	cap2a  = measureParasitic(area ( L_ALU1 ) two_net 0.500000e-4)
	cap2p  = measureParasitic(perimeter ( L_ALU1 ) two_net 0.900000e-4)
	cap3a  = measureParasitic(area ( L_ALU2 ) two_net 0.250000e-4)
	cap3p  = measureParasitic(perimeter ( L_ALU2 ) two_net 0.950000e-4)

	cap  = calculateParasitic( cap1a + cap1p + cap2a + cap2p + cap3a + cap3p )

	saveParasitic(cap   "PLUS" "MINUS" "c" "PCAPA symbol")
	)

;	saveInterconnect( ( ngate "L_NDIF") )
;	saveInterconnect( ( double "L_ALU1") )
;	saveInterconnect( ( pgate "L_PDIF") )
;	saveInterconnect( ( nsd "L_NDIF") )
;	saveInterconnect( ( psd "L_PDIF") )
;	saveInterconnect( ( pwell "L_PWELL") )
;	saveInterconnect( L_NDIF L_PDIF L_GATE )
;	saveInterconnect( L_REF_C L_NDIF L_PDIF L_GATE L_POLY L_CONT L_ALU1 L_VIA1 L_ALU2 )
	copyGraphics( ("text" "drawing" ))
	
/************************************************/
/*                                 	  	*/
/*    	     GATE EXTRACT DEFINITION		*/
/*                                   		*/
/************************************************/
	else

	    geomConnect( via( L_VIA1 L_ALU1 L_ALU2 ))
	    saveInterconnect( L_ALU1 L_ALU2 )
	)
)
/******************************************************************************/
/*                                                                            */
/*    	     FICHIER DE CONTROLE DES REGLES DE DESSIN                         */
/*           DESTINE AU DESIGN RULE CHECKING DE OPUS                          */
/*                                                                            */
/******************************************************************************/
 
    ivIf( switch("drc?") then

  	L_POLYGATE  = geomOr("L_POLY" "L_GATE")

; ***************************************************
; Verification des largeurs minimum des LAYOUTS
; ***************************************************
	
    drc( L_NWELL width < 4.0 "La largeur NWELL doit etre > 4.0 #1")
    drc( L_NTIE  width < 2.0 "La largeur NTIE doit etre > 2.0 #2")
    drc( L_PTIE  width < 2.0 "La largeur PTIE doit etre > 2.0 #3")
    drc( L_NDIF  width < 2.0 "La largeur NDIF doit etre > 2.0 #4")
    drc( L_PDIF  width < 2.0 "La largeur PDIF doit etre > 2.0 #5")
    drc( L_GATE  width < 1.0 "La largeur GATE doit etre > 1.0 #6")
    drc( L_POLY  width < 1.0 "La largeur POLY doit etre > 1.0 #7")
    drc( L_ALU1  width < 1.0 "La largeur ALU1 doit etre > 1.0 #8")
    drc( L_ALU2  width < 2.0 "La largeur ALU2 doit etre > 2.0 #9")
    drc( L_CONT  width < 1.0 "La largeur CONT doit etre > 1.0 #10")
    drc( L_VIA1  width < 1.0 "La largeur VIA doit etre > 1.0 #11")
    drc( L_POLYGATE  width < 1.0 "La largeur POLYGATE doit etre > 1.0 #7bis")

; ***************************************************
; Verification de la non intersection entre LAYOUTS
; ***************************************************
 
    saveDerived( geomStraddle( L_PTIE L_NWELL ) "Intersection PTIE-NWELL interdite  #12")
    saveDerived( geomStraddle( L_PTIE L_NTIE  ) "Intersection PTIE-NTIE  interdite  #13")
    saveDerived( geomStraddle( L_NDIF L_NWELL ) "Intersection NDIF-NWELL interdite  #14")
    saveDerived( geomStraddle( L_NDIF L_NTIE  ) "Intersection NDIF-NTIE  interdite  #15") 
    saveDerived( geomStraddle( L_NDIF L_PTIE  ) "Intersection NDIF-PTIE  interdite  #16") 
    saveDerived( geomStraddle( L_NTIE L_PDIF  ) "Intersection NTIE-PDIF  interdite  #17") 
    saveDerived( geomStraddle( L_PTIE L_PDIF  ) "Intersection PTIE-PDIF  interdite  #18") 
    saveDerived( geomStraddle( L_NDIF L_PDIF  ) "Intersection NDIF-PDIF  interdite  #19") 
    saveDerived( geomStraddle( L_GATE L_NTIE  ) "Intersection GATE-NTIE  interdite  #20") 
    saveDerived( geomStraddle( L_GATE L_PTIE  ) "Intersection GATE-PTIE  interdite  #21") 
    saveDerived( geomStraddle( L_POLY L_NTIE  ) "Intersection POLY-NTIE  interdite  #22") 
    saveDerived( geomStraddle( L_POLY L_PTIE  ) "Intersection POLY-PTIE  interdite  #23") 
    saveDerived( geomStraddle( L_POLY L_NDIF  ) "Intersection POLY-NDIF  interdite  #24") 
    saveDerived( geomStraddle( L_POLY L_PDIF  ) "Intersection POLY-PDIF  interdite  #25") 
    saveDerived( geomStraddle( L_CONT L_GATE  ) "Intersection CONT-GATE  interdite  #26") 
    saveDerived( geomStraddle( L_VIA1 L_GATE  ) "Intersection VIA-GATE   interdite  #27")
    saveDerived( geomStraddle( L_POLY L_VIA1  ) "Intersection VIA-POLY   interdite  #28")
    saveDerived( geomStraddle( L_VIA1 L_CONT   ) "Intersection VIA-CONT   interdite  #29")

; ****************************************************
; Verification de l'intersection minimum entre LAYOUTS
; ****************************************************

    drc( "L_NWELL" L_NWELL  ovlp < 4.0 "Intersection minimum NWELL doit etre > 4.0 #30")
    drc( "L_NTIE"  L_NTIE   ovlp < 2.0 "Intersection minimum NTIE doit etre > 2.0 #31")
    drc( "L_PTIE"  L_PTIE   ovlp < 2.0 "Intersection minimum PTIE doit etre > 2.0 #32")
    drc( "L_NDIF"  L_NDIF   ovlp < 2.0 "Intersection minimum NDIF doit etre > 2.0 #33")
    drc( "L_PDIF"  L_PDIF   ovlp < 2.0 "Intersection minimum PDIF doit etre > 2.0 #34")
    drc( "L_GATE"  L_GATE   ovlp < 1.0 "Intersection minimum GATE doit etre > 1.0 #35")
;   drc( "L_POLY"  L_GATE   ovlp < 1.0  "Intersection minimum POLY GATE doit etre > 1.0 #36")
    drc( "L_POLY"  L_POLY   ovlp < 1.0 "Intersection minimum POLY doit etre > 1.0 #37")
    drc( "L_ALU1"  L_ALU1   ovlp < 1.0 "Intersection minimum ALU1 doit etre > 1.0 #38")
    drc( "L_ALU2"  L_ALU2   ovlp < 2.0 "Intersection minimum ALU2 doit etre > 2.0 #39")


; ****************************************************
; Verification des  distances minimum entre LAYOUTS
; ****************************************************

     drc( L_NWELL         sep < 12.0 "Distance minimum entre NWELL doit etre > 12.0 #40")
     drc( L_NWELL L_NTIE  sep < 0.0  "Distance minimum entre NWELL NTIE doit etre > 0.0  #41")
     drc( L_NTIE          sep < 3.0  "Distance minimum entre NTIE doit etre > 3.0  #42")
     drc( L_NWELL L_PTIE  sep < 7.5  "Distance minimum entre NWELL PTIE doit etre > 7.5 #43")
     drc( L_NTIE  L_PTIE  sep < 8.0  "Distance minimum entre NTIE  PTIE doit etre > 8.0   #44")
     drc( L_PTIE          sep < 3.0  "Distance minimum entre PTIE doit etre > 3.0  #45")
     drc( L_NWELL L_NDIF  sep < 7.5  "Distance minimum entre NWELL NDIF doit etre > 7.5 #46")
     drc( L_NTIE  L_NDIF  sep < 8.0  "Distance minimum entre NTIE  NDIF doit etre > 8.0   #47")
     drc( L_PTIE  L_NDIF  sep < 3.0  "Distance minimum entre PTIE  NDIF doit etre > 3.0   #48")
     drc( L_NDIF          sep < 3.0  "Distance minimum entre NDIF doit etre > 3.0  #49")
     drc( L_NWELL L_PDIF  sep < 0.0  "Distance minimum entre NWELL PDIF doit etre > 0.0   #50")
     drc( L_NTIE  L_PDIF  sep < 3.0  "Distance minimum entre NTIE  PDIF doit etre > 3.0   #51")
     drc( L_PTIE  L_PDIF  sep < 8.0  "Distance minimum entre PTIE  PDIF doit etre > 8.0   #52")
     drc( L_NDIF  L_PDIF  sep < 8.0  "Distance minimum entre NDIF  PDIF doit etre > 8.0   #53")
     drc( L_PDIF          sep < 3.0  "Distance minimum entre PDIF doit etre > 3.0  #54")
     drc( L_NTIE  L_GATE  sep < 1.0  "Distance minimum entre NTIE  GATE doit etre > 1.0   #55")
     drc( L_PTIE  L_GATE  sep < 1.0  "Distance minimum entre PTIE  GATE doit etre > 1.0   #56")
     drc( L_NDIF  L_GATE  sep < 1.0  "Distance minimum entre NDIF  GATE doit etre > 1.0   #57")
     drc( L_PDIF  L_GATE  sep < 1.0  "Distance minimum entre PDIF  GATE doit etre > 1.0   #58")
     drc( L_GATE          sep < 2.0  "Distance minimum entre GATE doit etre > 2.0  #59")
     drc( L_NTIE  L_POLY  sep < 1.0  "Distance minimum entre NTIE  POLY doit etre > 1.0   #60")
     drc( L_PTIE  L_POLY  sep < 1.0  "Distance minimum entre PTIE  POLY doit etre > 1.0   #61")
     drc( L_NDIF  L_POLY  sep < 1.0  "Distance minimum entre NDIF  POLY doit etre > 1.0   #62")
     drc( L_PDIF  L_POLY  sep < 1.0  "Distance minimum entre PDIF  POLY doit etre > 1.0   #63")
;    drc( L_GATE  L_POLY  sep < 2.0  "Distance minimum entre GATE  POLY doit etre > 2.0   #64")
     drc( L_POLY          sep < 2.0  "Distance minimum entre POLY doit etre > 2.0  #65")
     drc( L_ALU1          sep < 2.5  "Distance minimum entre ALU1 doit etre > 2.5  #66")
     drc( L_ALU2          sep < 2.0  "Distance minimum entre ALU2 doit etre > 2.0    #67")
     drc( L_CONT          sep < 3.0  "Distance minimum entre CONT doit etre > 3.0    #68")
     drc( L_GATE  L_VIA1  sep < 2.0  "Distance minimum entre GATE  VIA  doit etre > 2.0   #69")
     drc( L_POLY  L_VIA1  sep < 2.0  "Distance minimum entre POLY  VIA  doit etre > 2.0   #70")
     drc( L_CONT  L_VIA1  sep < 2.0  "Distance minimum entre CONT  VIA  doit etre > 2.0   #71")
     drc( L_VIA1          sep < 3.0  "Distance minimum entre VIA  doit etre > 3.0   #72")
     drc( L_POLYGATE  L_POLYGATE  sep < 2.0  "Distance minimum entre GATE  POLY doit etre > 2.0   #64")


; ****************************************************
; Verification du recouvrement minimum entre LAYOUTS
; ****************************************************

     drc( L_NWELL L_NTIE  enc < 0.5 "Recouvrement minimum entre NWELL et NTIE doit etre > 0.5 #74")
     drc( L_NWELL L_PDIF  enc < 0.5 "Recouvrement minimum entre NWELL et PDIF doit etre > 0.5 #75")

     )
)
