/***********************************************************************************\

	pat2stl.c Matthieu Barret 22 Nov 1996
	Ce programme permet de transformer un .pat format alliance 
	en un .stl format Cadence.
	Cette version decoupe toujours les bus

\***********************************************************************************/



#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

struct pin
{
char *type;
char *name;
int  max;
int  min;
char bin;
int ok;
struct pin *suivant;
};

/************************************************************************************/

void erreur(char *err)
{
	printf("%s",err);
	exit(1);
}

/************************************************************************************\
	Procedure de nettoyage de la structure
\************************************************************************************/

void clean(struct pin *tmp,char *spat,FILE *stl)
{
struct pin *tmp2;
	while(tmp!=NULL)
	{
		free(tmp->type);
		free(tmp->name);
		tmp2=tmp;
		tmp=tmp->suivant;
		free(tmp2);
	}
	free(spat);
	fclose(stl);
}

/************************************************************************************\
	Procedure la pin est elle dans la liste interdite 
\************************************************************************************/

int isdel(char *liste,char *name)
{
char *res;
char *name2;

if (liste==NULL) return 1;

name2=(char *)malloc(sizeof(char)*(strlen(name)+3));
strcpy(name2,"/");
strcat(name2,name);
strcat(name2,"/");

res=strstr(liste,name2);
free(name2);
if (res==NULL) 	return 1;
else 		return 0;
}

/************************************************************************************\
	Procedure de chargement d'un fichier dans une chaine de caractere 
\************************************************************************************/

char *load_file(char *name)
{
FILE *file;
struct stat buf;
char *pos,*file_string;
long size=0;
long cpt;

file=fopen(name,"r");
if(file==NULL)
{
	printf("Cannot open %s in read mode\n",name);
 	exit(1);
}

stat(name,&buf);
size=buf.st_size;

if((file_string=(char *)malloc(sizeof(char)*size))==NULL)
	erreur("Not enough memory\n");

pos=file_string;
for(cpt=0;cpt<size;cpt++)
	*(pos++)=fgetc(file);
*(pos)='\0';
fclose(file);
return file_string;
}

/************************************************************************************\
	Procedure de nettoyage de la structure
\************************************************************************************/

void main(int argc,char **argv)
{
	FILE *stl;
	struct pin *first,*current,*last;
	char *spat,*pos,*liste;
	char *patname,*stlname;
	char *buff,*fin;
	int cpt,cpt2;
	long max,min;
	int value;
	int ok;
	int nb_pin=0;
	int nb_pattern=0;

	buff=(char *)malloc(sizeof(char)*10);

/* Initialisation des fichiers */
	if(argc>4||argc<3)
	{
 		printf("pat2vst : ");
 		if(argc>4) printf("Too many ");
     			else   printf("Missing ");
		printf("file arguments\n");
 		printf("Usage : pat2stl source target [exclusion pin]\n");
		printf("Where exclusion pin is a list of pin to exclude in the stl separated with '/'\n");
		printf("Exemple /vdd/vss/\n");
		exit(0);
	}

	patname=strdup(argv[1]);
	spat=load_file(patname);
	stlname=strdup(argv[2]);

	stl=fopen(stlname,"w");
	if(stl==NULL) 
	{
		printf("Cannot open %s in write mode\n",stlname);
		free(spat);
 		exit(1);
	}

	pos=spat;
	while(*pos=='\n'||(*pos=='-'&&*(pos+1)=='-'))
		pos=strstr(pos,"\n")+1;

/* Initialisation de la liste des pins a enlever si elle existe */

	if(argc==4)
		liste=strdup(argv[3]);
	else 	liste=NULL;

/* Initialisation du fichier stl */

	fprintf(stl,"; Created by pat2stl\n");
	fprintf(stl,"; ---STL Source Program Template---\n\n");
	fprintf(stl,"stlinit\n\n");
	fprintf(stl,"; note that clock inputs should be of type \"clk\"\n");
	first=NULL;
	current=NULL;

/* Lecture de la liste des pins et gestion des exclusions */
	fin=strstr(pos,"begin");
	if (fin==NULL) erreur("Begin of pattern cannot be found"); 

   	do
      	{
		last=current;
		current=(struct pin *)malloc(sizeof(struct pin));

		if(current==NULL) erreur("Failed initialise pin struct");

		if(first==NULL) first=current;
		else 		last->suivant=current;

		current->suivant=NULL;

 		current->type=(char *)malloc(sizeof(char)*20);
 		if(current->type==NULL) erreur("Not enough memory");

 		current->name=(char *)malloc(sizeof(char)*20);
 		if(current->name==NULL) erreur("Not enough memory");
		
		sscanf(pos,"%s %s",current->type,current->name);

		current->ok=isdel(liste,current->name);

		if(!strcmp(current->type,"inout")) strcpy(current->type,"io");
      		pos=strstr(pos+6,current->name)+strlen(current->name)+1;
      		sscanf(pos,"%c",&current->bin);
		if (current->ok)
     			if(current->bin=='(')
      			{
				sscanf(pos+1,"%ld %s %ld",&current->max,buff,&current->min);
				pos=strstr(pos,")")+2;
				sscanf(pos,"%c",&current->bin);
				for(cpt=current->min;cpt<=current->max;cpt++)
				{
				fprintf(stl,"defpin %s%d	%s\n",current->name,cpt,current->type);
				nb_pin++;
				}
			}
			else
			{
				current->max=current->min=0;
				fprintf(stl,"defpin %s	%s\n",current->name,current->type);
				nb_pin++;
			}
	do
		pos=strstr(pos,"\n")+1;
	while(*pos=='\n'||(*pos=='-'&&*(pos+1)=='-'));
        }
 	while(pos<fin);

	
	fprintf(stl,"\nstlMaxSimCompareMismatches = 20000\n");
	fprintf(stl,"\ndeftiming 1e-10 1e-07 2e-07  ; define basic time units\n");
	fprintf(stl,"\ndefformat ");
	current=first;
	cpt2=10;

/* Creation du defformat */

	while(current!=NULL)
	{
	if(current->ok)
		if(current->max!=current->min)
			for(cpt=current->min;cpt<=current->max;cpt++)
				{
				if(cpt2>70) { fprintf(stl,"\\\n	"); cpt2=10; }
				fprintf(stl,"%s%d ",current->name,cpt);
				cpt2+=strlen(current->name)+2;
				}
		else
				{
				if(cpt2>70) { fprintf(stl,"\\\n	"); cpt2=10; }
				fprintf(stl,"%s ", current->name);
				cpt2+=strlen(current->name)+1;
				}
		current=current->suivant;
	} 

	fprintf(stl,"\n\ndeftest\n\n");

	pos=fin;
	do
		pos=strstr(pos,"\n")+1;
	while(*pos=='\n'||(*pos=='-'&&*(pos+1)=='-'));

	pos=strstr(pos,":")+2;

/* Creation du Deftest en fonction du format choisi et des exclusions */

	while(pos!=NULL)
	{
		nb_pattern++;
		current=first;
		fprintf(stl,"\nxv(");
		while(current!=NULL)
		{
			if(current->ok)
			{
			ok=1;
			if(!strcmp(current->type,"out"))
				{
				pos=strstr(pos,"?")+1;
				if(*pos=='*')
					{
					for(cpt=current->min;cpt<=current->max;cpt++)
						fprintf(stl,"? ");
					ok=0;
					}
				}

			if(!strcmp(current->type,"io"))
				{
				if (*pos==' ') pos++;
					if(*pos=='?')
						if(*(pos+1)!='*')
							fprintf(stl,"Z %c ",*(pos+1));
						else
							fprintf(stl,"Z ? ");
					else
						if(*pos=='*')
							fprintf(stl,"? X ");
						else
							fprintf(stl,"%c X ",*pos);
					ok=0;
				}	
	
			if( ok==1)
				{
				switch (current->bin)
					{
					case 'X' : sscanf(pos,"%x",&value);
						   break;
					case 'O' : sscanf(pos,"%o",&value);
						   break;
					case 'B' : sscanf(pos,"%d",&value);
						   break;
					}
				for(cpt=current->min;cpt<=current->max;cpt++)
					{
					fprintf(stl,"%d ",value%2);
					value/=2;
					}
				}
			}
			pos=strstr(pos," ")+1;
			current=current->suivant;
			}
		fprintf(stl,")");
		do
		{
			pos=strstr(pos,"\n");
			if(pos!=NULL) pos+=1;
		}
		while((*pos=='\n'||(*pos=='-'&&*(pos+1)=='-'))&&pos!=NULL);

		pos=strstr(pos,":");
		if (pos!=NULL) pos+=2;
	}
   fprintf(stl,"\n\nendtest\n");

/* Fin liberation des pointeurs */
	printf("Done %d Patterns of %d Pins\n",nb_pattern,nb_pin);
   clean(first,spat,stl); 
   exit(1);
}
