/******************************************************************************/
/*                                                                            */
/*    	     FICHIER DE CONTROLE DES REGLES DE DESSIN                         */
/*           DESTINE AU DESIGN RULE CHECKING DE OPUS                          */
/*                                                                            */
/******************************************************************************/

drcExtractRules(

  NWELL = geomOr("L_NWELL")
  PWELL = geomOr("L_PWELL") 
  NDIF  = geomOr("L_NDIF")
  PDIF  = geomOr("L_PDIF")
  NTIE  = geomOr("L_NTIE")
  PTIE  = geomOr("L_PTIE")
  POLY  = geomOr("L_POLY")
  GATE  = geomOr("L_GATE")
  TPOLY = geomOr("L_TPOLY")
  ALU1  = geomOr("L_ALU1")
  TALU1 = geomOr("L_TALU1")
  VIA   = geomOr("L_VIA1")
  ALU2  = geomOr("L_ALU2")
  TALU2 = geomOr("L_TALU2")
  ALU3  = geomOr("L_ALU3")
  TALU3 = geomOr("L_TALU3")
  CONT = geomOr("L_CONT")
  POLYGATE  = geomOr("L_POLY" "L_GATE")
 
; ***************************************************
; Verification des largeurs minimum des LAYOUTS
; ***************************************************

    drc( NWELL width < 4.0 "La largeur NWELL doit etre > 4.0 #1")
    drc( NTIE  width < 2.0 "La largeur NTIE doit etre > 2.0 #2")
    drc( PTIE  width < 2.0 "La largeur PTIE doit etre > 2.0 #3")
    drc( NDIF  width < 2.0 "La largeur NDIF doit etre > 2.0 #4")
    drc( PDIF  width < 2.0 "La largeur PDIF doit etre > 2.0 #5")
    drc( GATE  width < 1.0 "La largeur GATE doit etre > 1.0 #6")
    drc( POLY  width < 1.0 "La largeur POLY doit etre > 1.0 #7")
    drc( ALU1  width < 1.0 "La largeur ALU1 doit etre > 1.0 #8")
    drc( ALU2  width < 2.0 "La largeur ALU2 doit etre > 2.0 #9")
    drc( CONT  width < 1.0 "La largeur CONT doit etre > 1.0 #10")
    drc( VIA   width < 1.0 "La largeur VIA doit etre > 1.0 #11")
    drc( POLYGATE  width < 1.0 "La largeur POLYGATE doit etre > 1.0 #7bis")

; ***************************************************
; Verification de la non intersection entre LAYOUTS
; ***************************************************
 
    saveDerived( geomStraddle( PTIE NWELL ) "Intersection PTIE-NWELL interdite  #12")
    saveDerived( geomStraddle( PTIE NTIE  ) "Intersection PTIE-NTIE  interdite  #13")
    saveDerived( geomStraddle( NDIF NWELL ) "Intersection NDIF-NWELL interdite  #14")
    saveDerived( geomStraddle( NDIF NTIE  ) "Intersection NDIF-NTIE  interdite  #15") 
    saveDerived( geomStraddle( NDIF PTIE  ) "Intersection NDIF-PTIE  interdite  #16") 
    saveDerived( geomStraddle( NTIE PDIF  ) "Intersection NTIE-PDIF  interdite  #17") 
    saveDerived( geomStraddle( PTIE PDIF  ) "Intersection PTIE-PDIF  interdite  #18") 
    saveDerived( geomStraddle( NDIF PDIF  ) "Intersection NDIF-PDIF  interdite  #19") 
    saveDerived( geomStraddle( GATE NTIE  ) "Intersection GATE-NTIE  interdite  #20") 
    saveDerived( geomStraddle( GATE PTIE  ) "Intersection GATE-PTIE  interdite  #21") 
    saveDerived( geomStraddle( POLY NTIE  ) "Intersection POLY-NTIE  interdite  #22") 
    saveDerived( geomStraddle( POLY PTIE  ) "Intersection POLY-PTIE  interdite  #23") 
    saveDerived( geomStraddle( POLY NDIF  ) "Intersection POLY-NDIF  interdite  #24") 
    saveDerived( geomStraddle( POLY PDIF  ) "Intersection POLY-PDIF  interdite  #25") 
    saveDerived( geomStraddle( CONT GATE  ) "Intersection CONT-GATE  interdite  #26") 
    saveDerived( geomStraddle( VIA  GATE  ) "Intersection VIA-GATE   interdite  #27")
    saveDerived( geomStraddle( POLY VIA   ) "Intersection VIA-POLY   interdite  #28")
    saveDerived( geomStraddle( VIA CONT   ) "Intersection VIA-CONT   interdite  #29")

; ****************************************************
; Verification de l'intersection minimum entre LAYOUTS
; ****************************************************

    drc( "L_NWELL" NWELL  ovlp < 4.0 "Intersection minimum NWELL doit etre > 4.0 #30")
    drc( "L_NTIE"  NTIE   ovlp < 2.0 "Intersection minimum NTIE doit etre > 2.0 #31")
    drc( "L_PTIE"  PTIE   ovlp < 2.0 "Intersection minimum PTIE doit etre > 2.0 #32")
    drc( "L_NDIF"  NDIF   ovlp < 2.0 "Intersection minimum NDIF doit etre > 2.0 #33")
    drc( "L_PDIF"  PDIF   ovlp < 2.0 "Intersection minimum PDIF doit etre > 2.0 #34")
    drc( "L_GATE"  GATE   ovlp < 1.0 "Intersection minimum GATE doit etre > 1.0 #35")
;   drc( "L_POLY"  GATE   ovlp < 1.0  "Intersection minimum POLY GATE doit etre > 1.0 #36")
    drc( "L_POLY"  POLY   ovlp < 1.0 "Intersection minimum POLY doit etre > 1.0 #37")
    drc( "L_ALU1"  ALU1   ovlp < 1.0 "Intersection minimum ALU1 doit etre > 1.0 #38")
    drc( "L_ALU2"  ALU2   ovlp < 2.0 "Intersection minimum ALU2 doit etre > 2.0 #39")


; ****************************************************
; Verification des  distances minimum entre LAYOUTS
; ****************************************************

     drc( NWELL       sep < 12.0 "Distance minimum entre NWELL doit etre > 12.0 #40")
     drc( NWELL NTIE  sep < 0.0  "Distance minimum entre NWELL NTIE doit etre > 0.0  #41")
     drc( NTIE        sep < 3.0  "Distance minimum entre NTIE doit etre > 3.0  #42")
     drc( NWELL PTIE  sep < 7.5  "Distance minimum entre NWELL PTIE doit etre > 7.5 #43")
     drc( NTIE  PTIE  sep < 8.0  "Distance minimum entre NTIE  PTIE doit etre > 8.0   #44")
     drc( PTIE        sep < 3.0  "Distance minimum entre PTIE doit etre > 3.0  #45")
     drc( NWELL NDIF  sep < 7.5  "Distance minimum entre NWELL NDIF doit etre > 7.5 #46")
     drc( NTIE  NDIF  sep < 8.0  "Distance minimum entre NTIE  NDIF doit etre > 8.0   #47")
     drc( PTIE  NDIF  sep < 3.0  "Distance minimum entre PTIE  NDIF doit etre > 3.0   #48")
     drc( NDIF        sep < 3.0  "Distance minimum entre NDIF doit etre > 3.0  #49")
     drc( NWELL PDIF  sep < 0.0  "Distance minimum entre NWELL PDIF doit etre > 0.0   #50")
     drc( NTIE  PDIF  sep < 3.0  "Distance minimum entre NTIE  PDIF doit etre > 3.0   #51")
     drc( PTIE  PDIF  sep < 8.0  "Distance minimum entre PTIE  PDIF doit etre > 8.0   #52")
     drc( NDIF  PDIF  sep < 8.0  "Distance minimum entre NDIF  PDIF doit etre > 8.0   #53")
     drc( PDIF        sep < 3.0  "Distance minimum entre PDIF doit etre > 3.0  #54")
     drc( NTIE  GATE  sep < 1.0  "Distance minimum entre NTIE  GATE doit etre > 1.0   #55")
     drc( PTIE  GATE  sep < 1.0  "Distance minimum entre PTIE  GATE doit etre > 1.0   #56")
     drc( NDIF  GATE  sep < 1.0  "Distance minimum entre NDIF  GATE doit etre > 1.0   #57")
     drc( PDIF  GATE  sep < 1.0  "Distance minimum entre PDIF  GATE doit etre > 1.0   #58")
     drc( GATE        sep < 2.0  "Distance minimum entre GATE doit etre > 2.0  #59")
     drc( NTIE  POLY  sep < 1.0  "Distance minimum entre NTIE  POLY doit etre > 1.0   #60")
     drc( PTIE  POLY  sep < 1.0  "Distance minimum entre PTIE  POLY doit etre > 1.0   #61")
     drc( NDIF  POLY  sep < 1.0  "Distance minimum entre NDIF  POLY doit etre > 1.0   #62")
     drc( PDIF  POLY  sep < 1.0  "Distance minimum entre PDIF  POLY doit etre > 1.0   #63")
;    drc( GATE  POLY  sep < 2.0  "Distance minimum entre GATE  POLY doit etre > 2.0   #64")
     drc( POLY        sep < 2.0  "Distance minimum entre POLY doit etre > 2.0  #65")
     drc( ALU1        sep < 2.5  "Distance minimum entre ALU1 doit etre > 2.5  #66")
     drc( ALU2        sep < 2.0  "Distance minimum entre ALU2 doit etre > 2.0    #67")
     drc( CONT        sep < 3.0  "Distance minimum entre CONT doit etre > 3.0    #68")
     drc( GATE  VIA   sep < 2.0  "Distance minimum entre GATE  VIA  doit etre > 2.0   #69")
     drc( POLY  VIA   sep < 2.0  "Distance minimum entre POLY  VIA  doit etre > 2.0   #70")
     drc( CONT  VIA   sep < 2.0  "Distance minimum entre CONT  VIA  doit etre > 2.0   #71")
     drc( VIA         sep < 3.0  "Distance minimum entre VIA  doit etre > 3.0   #72")
     drc( POLYGATE  POLYGATE  sep < 2.0  "Distance minimum entre GATE  POLY doit etre > 2.0   #64")


; ****************************************************
; Verification du recouvrement minimum entre LAYOUTS
; ****************************************************

     drc( NWELL NTIE  enc < 0.5 "Recouvrement minimum entre NWELL et NTIE doit etre > 0.5 #74")
     drc( NWELL PDIF  enc < 0.5 "Recouvrement minimum entre NWELL et PDIF doit etre > 0.5 #75")

)

